//
//  USBHIDAppDelegate.m
//  USBHID
//
//  Created by Michael Dolinar on 12-05-02.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "USBHIDAppDelegate.h"      
#import "IOKit/hid/IOHIDManager.h" 

@implementation USBHIDAppDelegate

@synthesize window = _window;

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
    // Insert code here to initialize your application
    
    // Create an HID Manager 
    IOHIDManagerRef HIDManager = IOHIDManagerCreate(kCFAllocatorDefault, 
                                                    kIOHIDOptionsTypeNone);
    
    // Create a Matching Dictionary
    CFMutableDictionaryRef matchDict = CFDictionaryCreateMutable(kCFAllocatorDefault, 
                                                                 2, 
                                                                 &kCFTypeDictionaryKeyCallBacks, 
                                                                 &kCFTypeDictionaryValueCallBacks); 
    
    // Specify a device manufacturer in the Matching Dictionary
    CFDictionarySetValue(matchDict, 
                         CFSTR(kIOHIDManufacturerKey), 
                         CFSTR("www.ontrak.net")); 
    
    // Register the Matching Dictionary to the HID Manager
    IOHIDManagerSetDeviceMatching(HIDManager, matchDict); 
    
    // Register a callback for USB device detection with the HID Manager
    IOHIDManagerRegisterDeviceMatchingCallback(HIDManager, &Handle_DeviceMatchingCallback, NULL);
    // Register a callback fro USB device removal with the HID Manager
    IOHIDManagerRegisterDeviceRemovalCallback(HIDManager, &Handle_DeviceRemovalCallback, NULL);
    
    // Register the HID Manager on our app’s run loop
    IOHIDManagerScheduleWithRunLoop(HIDManager, CFRunLoopGetMain(), kCFRunLoopDefaultMode);
    
    // Open the HID Manager
    IOReturn IOReturn = IOHIDManagerOpen(HIDManager, kIOHIDOptionsTypeNone);
    if(IOReturn) NSLog(@"IOHIDManagerOpen failed.");  //  Couldn't open the HID manager! TODO: proper error handling
}






// New USB device specified in the matching dictionary has been added (callback function)
static void Handle_DeviceMatchingCallback(void *inContext, 
                                          IOReturn inResult, 
                                          void *inSender, 
                                          IOHIDDeviceRef inIOHIDDeviceRef){
    
    // Retrieve the device name & serial number
    NSString *devName = [NSString stringWithUTF8String:CFStringGetCStringPtr(IOHIDDeviceGetProperty(inIOHIDDeviceRef, CFSTR("Product")), kCFStringEncodingMacRoman)];
    
    NSString *devSerialNumber = [NSString stringWithUTF8String:CFStringGetCStringPtr(IOHIDDeviceGetProperty(inIOHIDDeviceRef, CFSTR("SerialNumber")), kCFStringEncodingMacRoman)];
    
    // Log the device Name, Serial Number & device count
    NSLog(@"\nONTRAK device added: %p\nModel: %@\nSerial Number:%@\nONTRAK device count: %ld",
          inIOHIDDeviceRef,
          devName,
          devSerialNumber,
          USBDeviceCount(inSender));
}





// USB device specified in the matching dictionary has been removed (callback function)
static void Handle_DeviceRemovalCallback(void *inContext, 
                                         IOReturn inResult, 
                                         void *inSender, 
                                         IOHIDDeviceRef inIOHIDDeviceRef){
    
    // Log the device ID & device count
    NSLog(@"\nONTRAK device removed: %p\nONTRAK device count: %ld", 
          (void *)inIOHIDDeviceRef, 
          USBDeviceCount(inSender));
    
    // TODO: make sure your application doesn't try to do anything with the removed device
}





// Counts the number of devices in the device set (incudes all USB devices that match our dictionary)
static long USBDeviceCount(IOHIDManagerRef HIDManager){
    
    // The device set includes all USB devices that match our matching dictionary. Fetch it.
    CFSetRef devSet = IOHIDManagerCopyDevices(HIDManager);
    
    // The devSet will be NULL if there are 0 devices, so only try to count the devices if devSet exists
    if(devSet) return CFSetGetCount(devSet);
    
    // There were no matching devices (devSet was NULL), so return a count of 0
    return 0;
}

@end
